﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                            clcInfo_Rogue.lua
--
--    Copyright (C) 2012 Deronda
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Lesser General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--  This file is for use with clcInfo by abija.  Some of the code was pulled
--  from existing clcInfo modules and the example module.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      Deronda (@ Stormreaver)
------------------------------------------------------------------------------
-- Description:
--   Global namespace for clcInfo_Rogue.  The purpose is to avoid repetition.
--   However, since this module generates icons and bars that update many times
--   per second, locals should be used whenever possible and table lookups
--   avoided to speed things up.
--------------------------------------------------------------------------------
if not clcInfo then return end       -- need clcInfo to run
if clcInfo_Rogue then return end     -- don't load if already loaded
local ENGLISH_CLASS_NAME = "ROGUE"
local _, englishClass  = UnitClass("player") -- only load for rogues
if englishClass ~= ENGLISH_CLASS_NAME then return end

-----------------NAMESPACE---------------------------------------
clcInfo_Rogue = {}
clcInfo_Rogue.VERSION = 1.0
clcInfo_Rogue.CLASS = ENGLISH_CLASS_NAME

-----------------ROGUE SPELLS------------------------------------
clcInfo_Rogue.spells = {
    --assassination
    ["Cold Blood"]          = 14177,
    ["Mutilate"]            = 1329,
    ["Overkill"]            = 58426,
    ["Vendetta"]            = 79140,
    --combat
    ["Adrenaline Rush"]     = 13750,
    ["Bandit's Guile"]      = 84654,
    ["Blade Flurry"]        = 13877,
    ["Killing Spree"]       = 51690,
    ["Revealing Strike"]    = 84617,
    ["Shallow Insight"]     = 84745,
    ["Moderate Insight"]    = 84746,
    ["Deep Insight"]        = 84747,
    --subtlety
    ["Hemorrhage"]          = 16511,
    ["Honor Among Thieves"] = 51699,
    ["Find Weakness"]       = 91023,
    ["Master of Subtlety"]  = 31223,
    ["Premeditation"]       = 14183,
    ["Preparation"]         = 14185,
    ["Shadow Dance"]        = 51713,
    ["Shadowstep"]          = 36554,
    --generic rogue
    ["Ambush"]              = 8676,
    ["Backstab"]            = 53,
    ["Deadly Poison"]       = 67711,
    ["Envenom"]             = 32645,
    ["Eviscerate"]          = 2098,
    ["Fury of the Destroyer"]=109949,
    ["Garrote"]             = 703,
    ["Recuperate"]          = 73651,
    ["Rupture"]             = 1943,
    ["Sinister Strike"]     = 1752,
    ["Slice and Dice"]      = 5171,
    ["Stealth"]             = 1784,
    ["Vanish"]              = 1856,
    ["Tricks of the Trade"] = 57934,
    --armor debuffs
    ["Corrosive Spit"]      = 35387,
    ["Tear Armor"]          = 50498,
    ["Expose Armor"]        = 8647,
    ["Faerie Fire"]         = 770,
    ["Sunder Armor"]        = 7386,
    --Bleed debuffs aside from hemorrhage
    ["Mangle - Bear"]       =33878,
    ["Mangle - Cat"]        =33876,
    ["Trauma"]              =46857, -- the debuff placed by Blood Frenzy
    ["Tendon Rip"]          =50271,
    ["Gore"]                =35290,
    ["Stampede"]            =57386,
    --Procs from items or enchants
    ["Landslide"]           =74245,
    ["Tier13 2P"]           =105849,
    ["Tricks of Time"]      =105864, -- name of Tier 13 2p proc
    ["Agile"]               =107947, -- name of Kiroptyric Sigil ability
    ["Combat Trance"]       =109717, -- name of Wrath of Unchaining proc
    ["DS Legendaries"]      =109960,
      -- DS Legendary procs
    ["Nightmare"]           =109955,
    ["Suffering"]           =109959,
    ["Shadows of the Destroyer"]    =109941,
    ["Fury of the Destroyer"]       =109949,
}
--localize spells
local spell_name
for key, value in pairs(clcInfo_Rogue.spells) do
    spell_name = GetSpellInfo(value)
    if not spell_name then spell_name = "Unknown Spell" end
    clcInfo_Rogue.spells[key] = { id = value, name = spell_name }
end

--Generic Rogue cds for all specs
clcInfo_Rogue.basic_cds = {
    ["Blind"]               = 2094,
    ["Cloak of Shadows"]    = 31224,
    ["Combat Readiness"]    = 74001,
    ["Dismantle"]           = 51722,
    ["Evasion"]             = 5277,
    ["Feint"]               = 1966,
    ["Gouge"]               = 1776,
    ["Kick"]                = 1766,
    ["Kidney Shot"]         = 408,
    ["Redirect"]            = 73981,
    ["Smoke Bomb"]          = 76577,
    ["Sprint"]              = 2983,
    ["Tricks of the Trade"] = clcInfo_Rogue.spells["Tricks of the Trade"].id,
    ["Vanish"]              = clcInfo_Rogue.spells["Vanish"].id,
}

-----------------EVENTS------------------------------------------
-- event handling.
clcInfo_Rogue.EventFrame = CreateFrame("Frame", "clcInfo_RogueFrame", UIParent)
clcInfo_Rogue.EventFrame:Hide()
clcInfo_Rogue.EventFrame:SetScript("OnEvent", function(self, event, ...)
    clcInfo_Rogue[event](clcInfo_Rogue, event, ...)
end)
-- To add an event, write a clcInfo_Rogue function to
-- handle it and then register the event.  For instance:
--[[
function clcInfo_Rogue.A_WOW_EVENT(self, event, ...)
    -- handle the event
end
clcInfo_Rogue.EventFrame:RegisterEvent("A_WOW_EVENT")
]]--


-----------------FUNCTIONS---------------------------------------
-- Configuration functions - performance not an issue
--

-- creates the mod name. Each spec needs its own module
function clcInfo_Rogue.make_modName(specname)
    return "____rogue_" .. specname
end

-- Each spec needs its own priority list of actions as well.
function clcInfo_Rogue.StoreQueue(spec, q)
    clcInfo_Rogue[spec .. "_q"] = q
end

function clcInfo_Rogue.GetQueue(spec)
    return clcInfo_Rogue[spec .. "_q"]
end

-- Create the class module table with the name in class_module_name
-- Return the module and the module name.
function clcInfo_Rogue:init_module(specname)
    local class_module_name = self.make_modName(specname)
    local class_module = clcInfo:RegisterClassModule(class_module_name)
    return class_module, class_module_name
end

-- update the actions based on the prio settings the user provides
-- set the queue for the given spec
function clcInfo_Rogue.UpdateQueue(actions, tdb, spec)
    local q = {}
    local check = {} -- used to check for duplicates
    for key, value in ipairs(tdb.priority) do
        if not check[value] then
            if value ~= "none" then
                if actions[value] then
                    table.insert(q, value)
                else
                    print("clcInfo_Rogue invalid action:" .. value)
                end
            end
        else
            print("Ignoring duplicate action:"..key..")"..value)
        end
    end
    clcInfo_Rogue.StoreQueue(spec, q) 
end

-- register module and use default settings if version has changed
-- returns the template db, the queue, and the get and set functions for the
-- module options screen
-- called by clcInfo when templates update
function clcInfo_Rogue:OnTemplatesUpdate(class_module_name, actions,
                                         tdb_default, spec)
    local tdb = clcInfo:RegisterClassModuleTDB(class_module_name, tdb_default)
    local GetTDB, SetTDB, GetPriority, SetPriority
    if tdb then
        if not tdb.version or tdb.version < self.VERSION then
            -- fix stuff
            clcInfo.AdaptConfigAndClean(class_module_name, tdb, tdb_default)
            tdb.version = self.VERSION
        end

        self.UpdateQueue(actions, tdb, spec)
        GetTDB = clcInfo_Rogue.newGetTDB(tdb)
        SetTDB = clcInfo_Rogue.newSetTDB(tdb)
        GetPriority = clcInfo_Rogue.newGetPriority(tdb)
        SetPriority = clcInfo_Rogue.newSetPriority(tdb, actions, spec)
    end

    return tdb, GetTDB, SetTDB, GetPriority, SetPriority
end

-- called by clcInfo when initializing
-- returns the character db and the get and set functions for the
-- module options screen
function clcInfo_Rogue:OnInitialize(class_module_name, db_default)
    local db = clcInfo:RegisterClassModuleDB(class_module_name, db_default)
    local GetDB = self.newGetDB(db)
    local SetDB = self.newSetDB(db)
    return db, GetDB, SetDB
end

-- initializes the template db and q based on command line arguments
function clcInfo_Rogue:CmdLine(tdb, args, actions, spec)
    -- add args to options
    local num = 0
    local MAX_PRIORITY = 0
    for key, action in pairs(actions) do
        MAX_PRIORITY = MAX_PRIORITY + 1
    end

    for i, arg in ipairs(args) do
        if actions[arg] then
            if num < MAX_PRIORITY then
                num = num + 1
                tdb.priority[num] = arg
            else
                print("clcInfo_Rogue too many cmdLine priorities.\nMax is:"
                      .. MAX_PRIORITY)
            end
        else -- unrecognized action
            print(arg .. " not a recognized action.  Ignored.")
        end
    end
    
    -- none on the rest
    for i = num + 1, MAX_PRIORITY do
        db.priority[i] = "none"
    end

    -- redo queue
    self.UpdateQueue(actions, tdb, spec)
    clcInfo:UpdateOptions()
end

-- checks to see if a glyph is socketed
function clcInfo_Rogue.HasGlyphSocketed(glyphSpellID)
    for i = 1, NUM_GLYPH_SLOTS do
        local _, _, _, socketedGlyphSpellID = GetGlyphSocketInfo(i)
        if socketedGlyphSpellID == glyphSpellID then
            return true
        end
    end
    return false
end

---FUNCTIONS: Frequently called------------------------------------------------
-- Ability Icon Functions -> keep these fast!
local GetTime = GetTime
local GetSpellCooldown = GetSpellCooldown
local GetComboPoints = GetComboPoints
local UnitAura = UnitAura
local GetQueue = clcInfo_Rogue.GetQueue
local clcInfo_debug = clcInfo.debug
local emod = clcInfo.env
local IconSpell = clcInfo.env.IconSpell
local gcdSpellId = clcInfo_Rogue.spells["Sinister Strike"].id

-- locals for keeping track of time and combo points
local current_time, combo_points
local gcd_duration = 0

--current ability id, next ability id
local cur_id, next_id
local cur_yellow, next_yellow --yellow icon = wait

-- gets the initial status: time, gcd situation, and combo points
local function SetStatus()
    cur_id = nil

    -- current time
    current_time = GetTime() 

    -- gcd value
    local start, gcd, gcd_left
    
    start, gcd = GetSpellCooldown(gcdSpellId)
    if gcd > 0 then
        gcd_duration = gcd
    end
    gcd_left = start + gcd_duration - current_time
    if gcd_left < 0 then gcd_left = 0 end
    current_time = current_time + gcd_left
    
    --combo points
    combo_points = GetComboPoints("player")
end

-- Given an aura id, determines if it's present on the unit.
-- Works just like UnitAura, but with a spell ID instead of a spell name.
-- This function is O(N), so avoid using when possible.
local function UnitAuraByID(unit, targetSpellID, filter)
    for i=1, MAX_TARGET_DEBUFFS do
        local name, rank, icon, count, dispelType, duration, expires, caster,
        isStealable, shouldConsolidate, spellID, canApplyAura, isBossDebuff,
        value1, value2, value3 = UnitAura(unit, i, filter)
        if spellID == targetSpellID then
            return name, rank, icon, count, dispelType, duration, expires,
                   caster, isStealable, shouldConsolidate, spellID,
                   canApplyAura, isBossDebuff, value1, value2, value3
        end 
    end
end

-- given a buff or debuff, the unit, and a filter, returns the time left based
-- on the current time and time left on the gcd
-- spell can be either a spell id or a spellname
-- returns 0 if no such buff or debuff is present
function clcInfo_Rogue.GetTimeLeft(unit, spell, filter)
    local time_left = 0
    local name, expires
    if type(spell) == "string" then 
        name, _, _, _, _, _, expires = UnitAura(unit, spell, nil, filter)
    else
        name, _, _, _, _, _, expires = UnitAuraByID(unit, spell, filter)
    end
    if name then
        time_left = expires - current_time
    end
    return time_left
end

-- given a spell id returns the cd left based on the current time variables
function clcInfo_Rogue.GetCooldown(id)
    local start, duration = GetSpellCooldown(id)
    local cd = start + duration - current_time
    if cd < 0 then return 0 end
    return cd
end

-- Checks the target for all bleed debuffs. Returns the maximum
-- time left.  For instance, if there is 45 seconds left on a
-- Hemorrhage and 7 left on Tendon Rip, it returns 45.
-- Returns 0 if no bleed debuff.
local MangleBear = clcInfo_Rogue.spells["Mangle - Bear"].name
local MangleCat = clcInfo_Rogue.spells["Mangle - Cat"].name
local Hemo = clcInfo_Rogue.spells["Hemorrhage"].name
local Trauma = clcInfo_Rogue.spells["Trauma"].name
local TendonRip = clcInfo_Rogue.spells["Tendon Rip"].name
local Gore = clcInfo_Rogue.spells["Gore"].name
local Stampede = clcInfo_Rogue.spells["Stampede"].name
local GetTimeLeft = clcInfo_Rogue.GetTimeLeft

function clcInfo_Rogue.BleedDebuffLeftOnTarget()
    return max(GetTimeLeft("target", MangleCat, "HARMFUL"),
               GetTimeLeft("target", MangleBear, "HARMFUL"),
               GetTimeLeft("target", Trauma, "HARMFUL"),
               GetTimeLeft("target", Hemo, "HARMFUL"),
               GetTimeLeft("target", TendonRip, "HARMFUL"),
               GetTimeLeft("target", Gore, "HARMFUL"),
               GetTimeLeft("target", Stampede, "HARMFUL"))
end

-- Calculates the target's health as a percentage.
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
function clcInfo_Rogue.CalculateTargetHealthPct()
    return floor(UnitHealth("target") / UnitHealthMax("target") * 100)
end

--check if stealthed, then remember it to shorten further action checks
--stealthed must be reset to nil before each GetNextAction() is performed
local ShadowDanceId = clcInfo_Rogue.spells["Shadow Dance"].id
local ShadowDance = clcInfo_Rogue.spells["Shadow Dance"].name
local Stealth = clcInfo_Rogue.spells["Stealth"].name
local VanishId = clcInfo_Rogue.spells["Vanish"].id
local Vanish = clcInfo_Rogue.spells["Vanish"].name
local stealthed

function clcInfo_Rogue.IsStealthed()
    if stealthed == nil then
        stealthed = (cur_id == ShadowDanceId or cur_id == VanishId 
                     or (UnitAura("player", Vanish))
                     or (UnitAura("player", Stealth))
                     or GetTimeLeft("player", ShadowDance, "HELPFUL") > 0)
    end
    return stealthed
end

-- figures out which ability should be used next
local function GetNextAction(actions, q)
    stealthed = nil
    local n = #q
    local action_index

    -- parse once, get cooldowns, return first 0
    for i = 1, n do
        action_index = q[i]
        local action = actions[action_index]
        local cd, yellow = action.GetCD(cur_id, combo_points)
        if clcInfo_debug.enabled then
            clcInfo_debug:AddBoth(action_index, cd)
        end
        if cd == 0 then
            return action.id, action_index, yellow
        end
        action.cd = cd
        action.yellow = yellow
    end
    
    -- parse again, return min cooldown
    local minQ = 1
    local minCd = actions[q[1]].cd
    for i = 2, n do
        local action_cd = actions[q[i]].cd
        if minCd > action_cd then
            minCd = action_cd
            minQ = i
        end
    end
    action_index = q[minQ]
    return actions[action_index].id, action_index, actions[action_index].yellow
end

-- determines the current and next abilities
local function RogueRotation(actions, q)
    SetStatus()
    local action
    cur_id, action, cur_yellow = GetNextAction(actions, q)
    combo_points, ignoreTimeIncrement=actions[action].UpdateStatus(combo_points)
    if not ignoreTimeIncrement then
        current_time = current_time + gcd_duration
    end
    next_id, action, next_yellow = GetNextAction(actions, q)
end

local nextSkill
local devName = clcInfo_Rogue.spells["Sinister Strike"].name

-- Get current ability and shade it yellow if the user should wait to use it.
function clcInfo_Rogue.IconCurrent(actions, spec, rangePerSkill)
    local q = GetQueue(spec)
    RogueRotation(actions, q)
    if nextSkill then nextSkill:DoUpdate() end
    if not cur_yellow then
        return IconSpell(cur_id, rangePerSkill or devName)
    end
    local visible, texture, start, duration, enable, reversed, count, alpha
           = IconSpell(cur_id, rangePerSkill or devName)
    return visible, texture, start, duration, enable, reversed, count, alpha,
           true, 0.8, 0.8, 0.1, 1
end

-- Determine Next ability and shade yellow if ser will have to wait to use it
local function NextExec()
    if not next_yellow then
        return IconSpell(next_id, rangePerSkill or devName)
    end
    local visible, texture, start, duration, enable, reversed, count, alpha
           = IconSpell(next_id, rangePerSkill or devName)
    return visible, texture, start, duration, enable, reversed, count, alpha,
             true, 0.8, 0.8, 0.1, 1    
end

-- reset next ability
local function ExecCleanupNext()
    nextSkill = nil
end

-- Get Next ability
function clcInfo_Rogue.IconNext()
    nextSkill = emod.___e
    nextSkill:SetScript("OnUpdate", nil)
    nextSkill.exec = NextExec
    nextSkill.ExecCleanup = ExecCleanupNext
end