﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                                options.lua
--
--    Copyright (C) 2012 Deronda
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Lesser General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--  This file is for use with clcInfo by abija.  Some of the code was pulled
--  from existing clcInfo modules and the example module.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      Deronda (@ Stormreaver)
------------------------------------------------------------------------------
-- Description:
--   Options for clcInfo_Rogue
------------------------------------------------------------------------------
if not clcInfo then return end --clcInfo must be loaded
if not clcInfo_Rogue then return end

local _, englishClass = UnitClass("player")
if englishClass ~= clcInfo_Rogue.CLASS then return end  --must be rogue

--each spec needs its own set and get functions to refer to their
--own db and tdb upvalues.  So we use closures for Get and Set functions for
--the AceGUI callbacks.
function clcInfo_Rogue.newGetDB(db)
    return function (info)
               return db[info[#info]]
           end
end

function clcInfo_Rogue.newSetDB(db)
    return function (info, val)
               db[info[#info]] = val
           end
end

function clcInfo_Rogue.newGetTDB(tdb)
    return function (info)
               return tdb[info[#info]]
           end
end

function clcInfo_Rogue.newSetTDB(tdb, actions, spec)
    return function (info, val)
               tdb[info[#info]] = val
           end
end

function clcInfo_Rogue.newGetPriority(tdb)
    return function (info)
               local i = tonumber(info[#info])
               return tdb.priority[i]
           end
end

function clcInfo_Rogue.newSetPriority(tdb, actions, spec)
    return function (info, val)
               local i = tonumber(info[#info])
               tdb.priority[i] = val
               clcInfo_Rogue.UpdateQueue(actions, tdb, spec)
           end
end

--Create a module options screen
function clcInfo_Rogue.LoadModule(class_module_name, talent_tree_name,
                                  sample_prio, actions, GetDB, SetDB, GetTDB,
                                  SetTDB, GetPriority, SetPriority)

    clcInfo_Options.options.args.classModules.args[class_module_name] = {
        order = 2, type = "group", childGroups = "tab", name = talent_tree_name,
        args = {
            tabPriority = {
                order = 1, type = "group", name = ABILITIES, args = {
                    igSettings = {
                        order=1, type = "group", inline = true, name = SETTINGS,
                        args = {
                            ignore_rupture_no_bleed = {
                                order = 1, type = "toggle", width = "full",
                                name = format("%s %s = %s %s", NO,
                                              string.sub(EMOTE10_CMD1, 2), 
                                NO, clcInfo_Rogue.spells["Rupture"].name),
                                get = GetTDB, set = SetTDB,
                            },
                            rangePerSkill = {
                                order = 2, type = "toggle", width = "full",
                                name = format("%s/%s", RANGED, SKILL),
                                get = GetTDB, set = SetTDB,
                            },
                        },
                    },
                    prioSettings = {
                        order = 2, type = "group", inline = true, name = RANK,
                        args = {} -- place holder for priorities
                    },
                },           
            },
        },
    }

    -- generate priorities interface (fill in the place holder!!)
    local spellChoice = { none = "None" }
    local num = 0
    for key, action in pairs(actions) do
        spellChoice[key] = action.info
        num = num + 1
    end

    local prioArgs = clcInfo_Options.options.args.classModules
                     .args[class_module_name].args.tabPriority.args
                     .prioSettings.args
    for i = 1, num do
        prioArgs[tostring(i)] = {
            order = i, type = "select", name = tostring(i), 
            get = GetPriority, set = SetPriority, values = spellChoice,
        }
    end
end